#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method.
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later

// *** Copyright ***
// © 2023 CNRS Shared by Pierre VINCENT, under the MIT License.
//	Contact: pierre.vincent@cnrs.fr; IGF, 141 rue de la Cardonille, 34094 Montpellier cedex 5, France.

//	Rewritten in Raplouf 9.10
//	Main change: to allow for ROI to "move" during the experiment, there was a complicated system with either default (immobile ROI) or moving.
//	This required to activate and inactivate dependencies
//	To make it easier, now, all ROI have the possibility to move. 
//	A reference time-point defines at which timepoint the ROI should be drawn. From that timepoint, the current ROI is drawn as the reference + movement wave
//	Each ROI is related to the following objects:
//	ROIx0$$$$		the x wave defining the ROI polygon, in image coordinates, for the reference time-point
//	ROIy0$$$$		the y wave defining the ROI polygon
//	ROIx$$$$		the x wave defining the ROI polygon, in image coordinates, to be displayed over the image at the current time-point
//	ROIy$$$$		the y wave defining the ROI polygon
//	ROIdxyz$$$$		defines the position difference (in xyz dimensions) between the reference datapoint and the currently displayed datapoint
//	The ROI is displayed over the image by plotting ROIy$$$$ vs ROIx$$$$. The currently displayed ROI is dependent on the reference ROI plus the movement:
//	ROIx$$$$ := ROIx0$$$$ + ROIdxyz$$$$
//	
//	For the measurement, it would be cool to apply the mask over the whole image, but doing a calculation on a whole image takes a lot of time.
//	Much faster to extract a sub-region (3D) from the image, and do the calculation on this extracted zone with a mask of the same size.
//	The zone is extracted at the current timepoint using the coordinates of the ROI + ROIdxyz$$$$
//	Masque2_$$$$	is the mask defined by the polygon, in ROI coordinates, for the reference timepoint.
//	TraceA$$$$, TraceB$$$$ and TraceR$$$$ are the waves holding the measurements for A, B channels, and ratio.


//________________________________________________________________________________________________
//************************************************************************************************
//	Several different analysis can be performed for a same experiment, each one in a different "Analyse_*" datafolder
// Analyse_* datafolder contains ROI definition, measurement traces, masks..., everything that pertains to the measurement

//	This function is activated by the selector on P_Controle, to select another set of ROIs for analysis
//	Also called when Raplouf is started, after P_Controle is created
//	When the function is called, there can be an analysis datafolder that is already selected in the popup (default is Analysis_0), that will change to another
//	Requires to close references to the former analysis and create new references to objects in the new analysis datafolder
Function PopMenuSelectAnalyse(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	variable i
	string sNomDF, 	sFormule

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	DFREF dfROIsav=W_DFREF[0]	//	current analysis df

	//	Clears references to objects in the current analysis folder
	//	Windows display objects contained in the current analysis datafolder.
	//	It's easier to kill the window and create a fresh one, which will then be created with references to the correct objects.
	KillWindow/Z P_ROI			//	kills the ROI control panel
	KillWindow/Z F_Pile	//	kills stack image; force update
	CheckBox/Z C_Active_P_ROI win=P_Controle,value=0	//	unchecks the corresponding box
	KillWindow/Z P_TableauROI			//	kills table with ROI descriptions
	CheckAfficheMoy("",0)				//	remove average traces

	//	Remove dependencies (if any) linking ROI objects in this datafolder
	NVAR/SDFR=dfROI V_NbROI
	For (i=1; (i<=V_NbROI); i+=1)
		DependencesROI(0, i)	//	Suppresses dependencies in mobile ROIs
	endfor

	//	Remove traces and ROI plots
	MaJApparenceToutesROI(0)	

	StrSwitch(popStr)
		//	make a new analysis datafolder
		case "New":
		case "Duplicate":
			sNomDF="Analyse_"
			Do		//	Ask the user for a non-existing new name
				prompt sNomDF, "Name of the datafolder for this new analysis"
				DoPrompt/Help="The name of this datafolder must be unique and must start with \"Analyse_\"" "Raplouf needs to know", sNomDF
			while ( (DataFolderExists(sNomDF)) || (CmpStr(sNomDF[0,7], "Analyse_")!=0) )	//	Repeat until a new name is provided that starts with "Analyse_"
			StrSwitch(popStr)
				case "New":
					NewDataFolder root:$sNomDF
				break
				case "Duplicate":
					DuplicateDataFolder dfROIsav, root:$sNomDF
				break
			EndSwitch
			dfROI = root:$sNomDF
		break
		case "Initialise":	//	called during InitialiseRaplouf; do nothing special here
		break
		default:	//	user has selected the name of an existing datafolder in the popup
			dfROI=root:$popStr
		break
	EndSwitch

	//	Save the new df ref in the proper wave
	//	Now that W_DFREF is set, all further operations will work on this datafolder
	W_DFREF[0]=dfROI
	sNomDF=GetDataFolder(0, dfROI)

	//	Make sure the resulting Analyse_* datafolder contains all required up-to-date object	
	InitDataFolder("Init raplouf.pxp", "f_Analyse", sNomDF)	//	initializes content of this df with required objects
	
	//	Some specific changes to a few objects
	Redimension/B dfROI:W_Tag, dfROI:W_PlanROI, dfROI:W_AfficheROI, dfROI:W_AfficheTrace, dfROI:W_IntensiteOK	//	8 bit integer is enough
	Redimension/S dfROI:W_EpaisseurTrace, dfROI:W_EpaisseurROI	//	usefull ?
	NVAR/SDFR=dfROI V_NbROI
	Wave/SDFR=dfROI W_ZonesROI
	Redimension/W/N=(-1, V_NbROI+1) W_ZonesROI
	//	saves color global definitions in current df: this allows to kill the z_Initialisation df when saving.
	Duplicate/O root:z_Initialisation:f_Global:W_CouleurTagROI, dfROI:W_CouleurTagROI
	Duplicate/O root:z_Initialisation:f_Global:W_CouleurTagTraces, dfROI:W_CouleurTagTraces
	
	//	During initialization of old files, may need to update the nb of ROI and resize W_ZonesROI accordingly
	//	Make sure the number of ROI is correct
	NbROI()
	Redimension/N=(-1, V_NbROI+1) W_ZonesROI
	
	//	Now that the objects in Analyse_* are created, they can be accessed
	NVAR/SDFR=dfROI V_NumROI, V_Position0DeplaceROI, V_NbROI, V_EpaisseurMoy
	for (V_NumROI=1; (V_NumROI<=V_NbROI); V_NumROI+=1)
		BoutonGereROI("")	//	-> creation of movement wave (if missing), masks, then -> ROI display update
	endfor
	//	at the end of previous loop, V_NumROI=V_NbROI+1 => reset to first ROI
	//	if V_NbROI==0, which is the case when this analyse_* is started, V_NumROI MUST remain =0
	V_NumROI = V_NumROI == 0 ? 0 : 1
	
	MaJApparenceToutesROI(1)				//	display traces and ROIs -> makes masks and creates dependencies

	Init_P_Controle()	//	connects controls in P_Control with the correct objects in Analyse_*

	//	Sets the controls to the correct value
	sFormule="root:"+sNomDF+":W_Fond[root:Ratio:V_NumCoucheCourante][0]"
	ValDisplay C_FondAA, win=P_Controle, value=#sFormule
	sFormule="root:"+sNomDF+":W_Fond[root:Ratio:V_NumCoucheCourante][1]"
	ValDisplay C_FondBB, win=P_Controle, value=#sFormule
	SetVariable C_EpaisseurMoyenne, win=P_Controle, value=V_EpaisseurMoy

End

Function/S ListeDFAnalyse()
	String sListe="New;Duplicate;", sNomObj
	variable i
	Do
		sNomObj=GetIndexedObjName("root:",4, i)	
		if (strlen(sNomObj) == 0)
			break
		endif
		if (CmpStr(sNomObj[0,7],"Analyse_")==0)
			sListe+=sNomObj
			sListe+=";"
		endif
		i+=1
	while(1)
	return(sListe)
end

//________________________________________________________________________________________________
//************************************************************************************************
// *** Defining a ROI ***
//	Main function that controls proper ROI definition, dependencies. 
//	Called by InitAnalyse to ensure that all ROI objects are defined and dependencies are activated
//	Responds to buttons on P_ROI for making, modifying... ROIs
//	Affects selectively the ROI identified by the global V_NumROI
//	Makes sure that all waves pertaining to this ROI are present
//	Creates the ROI if called by "C_DessineROI"
//	Copies ROI description into ROIx0_ and ROIy0_
//	Calculates all parameters of the ROI (mask, morphometry)
//	Does not contain the code for controlling appearance, but instead, calls MaJApparenceROI to do that

Function BoutonGereROI(ctrlName) : ButtonControl
	String ctrlName
	
	//	Stops auto-update
	CheckAjoutePointsAuto("",0)

	MajApparenceToutesROI(2)	//	updates traces' appearance

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	DFREF dfSav=getdatafolderdfr()
	SetDataFolder dfROI
	Wave W_PlanROI, W_AfficheROI, W_ZonesROI, W_ROI_Area, W_ROI_EllipseData, D_Masque
	NVAR V_NumROI, V_NbROI, V_Position0DeplaceROI
	NVAR/SDFR=dfRatio V_NbChiffres, kMax, V_NumPlan, V_NumCoucheCourante, V_Resolution

	//	If ROIs are mobile (a value was set to V_Position0DeplaceROI), go to this reference timepoint
	if (NumType(V_Position0DeplaceROI)==0)
		V_NumCoucheCourante=V_Position0DeplaceROI
		MaJImages(0)		//	to display the datapoint corresponding to V_Position0DeplaceROI timepoint
	endif

	//	When adding a new ROI, this must come first
	StrSwitch(ctrlName)
		case "C_DessineROI":	//	create a new ROI
			V_NbROI+=1
			V_NumROI=V_NbROI		//	Updates ROI # to the last
			Redimension/N=(-1, V_NbROI+1) W_ZonesROI
			SetVariable/Z C_NumROI,limits={1,V_NbROI,1},win=P_Controle
			SetVariable/Z C_NumROI,limits={1,V_NbROI,1},win=P_ROI
		break
	EndSwitch

	//	Defining the name of all objects, based on the ROI number
	String NomROIx=AjouteNumero("ROIx", V_NumROI, V_NbChiffres)
	String NomROIx0=AjouteNumero("ROIx0_", V_NumROI, V_NbChiffres)
	String NomROIy=AjouteNumero("ROIy", V_NumROI, V_NbChiffres)
	String NomROIy0=AjouteNumero("ROIy0_", V_NumROI, V_NbChiffres)
	String NomTraceA=AjouteNumero("TraceA",V_NumROI, V_NbChiffres)
	String NomTraceB=AjouteNumero("TraceB",V_NumROI, V_NbChiffres)
	String NomTraceR=AjouteNumero("TraceR",V_NumROI, V_NbChiffres)
	String NomMasque2=AjouteNumero("Masque2_",V_NumROI, V_NbChiffres)
	String NomROIdxyz=AjouteNumero("ROIdxyz", V_NumROI, V_NbChiffres)	

	StrSwitch(ctrlName)
		case "C_DessineROI":	//	to interactively draw a ROI
		case "C_EditeROI":		//	to interactively modify an existing ROI
		//	Store the layer number corresponding to this ROI
			W_PlanROI[V_NumROI]=V_NumPlan

			//	Create modal window
			NewPanel /K=2 /W=(20,20,220,90) as "Create or edit a ROI"	//	create the panel
			DoWindow/C F_DefineROI	//	give it a name
			AutoPositionWindow/M=0 /R=F_RM	//	position it next to the image window
			DrawText 10, 20, "Draw ROI on the image"	//	build the content of the window
			Button C_BoutonValide,pos={15,30},size={170,20},proc=BoutonValideROI,title="Validate ROI"
			DoWindow/F F_RM	//	put the image window to the forefront
			StrSwitch(ctrlName)
				case "C_DessineROI":
					Make/O/N=(kMax) dfROI:$NomTraceA=NAN, dfROI:$NomTraceB=NAN, dfROI:$NomTraceR=NAN
					Make/O/N=(kMax,3) dfROI:$NomROIdxyz=0
					Make/O/N=0 dfROI:$NomROIy, dfROI:$NomROIx
					GraphWaveDraw/W=F_RM/O/L/T $NomROIy, $NomROIx 			//	Plots the wave and enters interactive edit mode for this wave
					ModifyGraph rgb($nomROIy)=(65535,0,50000)
				break
				case "C_EditeROI":
					//	Clear dependencies, so that the user can edit it
					DependencesROI(0, V_NumROI)
					//	Interactively edit the already plotted ROI waves
					GraphWaveEdit $NomROIy
				break
			EndSwitch
			PauseForUser F_DefineROI, F_RM	//	Stops interaction with any window other than these two in the forefront
		break
		case "C_RAZRoi":
			DoAlert/T="Warning !" 1, "Reset this ROI ?"
			if (V_Flag==1)
				Wave wTraceA=$NomTraceA; wTraceA=NAN
				Wave wTraceB=$NomTraceB; wTraceB=NAN
				Wave wTraceR=$NomTraceR; wTraceR=NAN

				Wave wX=$NomROIx, wY=$NomROIy
				Redimension/N=2 wX, wY
				//	Set values so that it goes as a line across the displayed area
				GetAxis/W=F_RM/Q top
				wX[0]=trunc(V_Min+0.2*(V_max-V_min))
				wX[1]=trunc(V_Min+0.8*(V_max-V_min))
				GetAxis/W=F_RM/Q left
				wY[0]=trunc(V_Min+0.8*(V_max-V_min))
				wY[1]=trunc(V_Min+0.2*(V_max-V_min))
				W_PlanROI[V_NumROI]=0
				W_AfficheROI[V_NumROI]=1
				NbROI()
			endif
		break
	EndSwitch
	
	Wave wX=$NomROIx, wY=$NomROIy
	Wave/Z wX0=$NomROIx0, wY0=$NomROIy0, wDXYZ=$NomROIdxyz

	//	analysis files older than version 9.10 don't have a movement or ref ROI waves;	make sure these waves are defined; if yes, keep it; if not, make a new one
	if (!WaveExists(wDXYZ))
		Make/N=(kMax,3) $NomROIdxyz	//	created with value set to 0, meaning no movement
		Wave wDXYZ=$NomROIdxyz
	endif
	If (!WaveExists(wX0))
		Duplicate wX, $NomROIx0
		Duplicate wY, $NomROIy0
	endif
	Wave wX0=$NomROIx0, wY0=$NomROIy0

	//	Copy ROIx and ROIy in ROIx0 and ROIy0
	Duplicate/O wX, wX0
	Duplicate/O wY, wY0
	
	//	Make the dependency link only if there is a movement in the wave
	WaveStats/M=1/Q wDXYZ
	if ((V_min!=0) || (V_max!=0))	//	check if there is a non-zero value in the movement wave
		DependencesROI(1, V_NumROI)
	endif
	
	//	Saves the boundaries of the ROI
	Variable vX0, vX1, vY0, vY1
	WaveStats/M=1/Q wX
	vX0=trunc(V_min)
	W_ZonesROI[0][V_NumROI]=vX0
	vX1=trunc(V_max)
	W_ZonesROI[3][V_NumROI]=vX1
	WaveStats/M=1/Q wY
	vY0=trunc(V_min)
	W_ZonesROI[1][V_NumROI]=vY0
	vY1=trunc(V_max)
	W_ZonesROI[4][V_NumROI]=vY1
	//	with 2D measurement, Z is the plane associated with the ROI
	W_ZonesROI[2][V_NumROI]=W_PlanROI[V_NumROI]
	W_ZonesROI[5][V_NumROI]=W_PlanROI[V_NumROI]+1

	//	Duplicate ROI waves and shifts them to the upper left corner, to make the smallest possible rectangle enclosing the mask
	Duplicate/O wX, wX_mask
	wX_mask -= vX0
	Duplicate/O wY, wY_mask
	wY_mask -= vY0

// convert ROI to mask
	Make/O/N=(vX1-vX0, vY1-vY0) D_MasqueTemp
	NewImage/k=1/N=F_Masque D_MasqueTemp
	SetDrawLayer/W=F_Masque ProgFront
	SetDrawEnv/W=F_Masque xcoord=top, ycoord=left
	DrawPoly/W=F_Masque/ABS 0,0,1,1, wX_mask, wY_mask
	ImageGenerateROIMask/W=F_Masque/E=1/I=0 D_MasqueTemp
	Wave M_ROIMask
	Duplicate/O M_ROIMask, dfROI:$NomMasque2
	KillWindow F_Masque
	DoUpdate
	KillWaves D_MasqueTemp, M_ROIMask, wX_mask, wY_mask
	
	//	Update global functioning	
	MaJ_CasesACocher("Mesure", 0, "", "")	//	ROI must be measured after this change
	W_AfficheROI[V_NumROI]=1
	NbROI()
	DenombreGroupe()
	MajApparenceToutesROI(1)	//	updates traces' presence on the graph and appearance

	//	Update ROI display in P_ROI
	SetVarNumROI("", 0, "", "")

	//	Measure "intensity" (estimate of sensor concentration) over this ROI
	MesureIntensite(V_NumROI)

// *** ROI morphometry ***
	Wave wMasque=$NomMasque2
	NewDataFolder/O/S Ellipse	//	do all the ellipse measurement in a sub-datafolder to reduce the clutter
	ImageAnalyzeParticles/E/Q stats wMasque	//	Fitting an ellipse to the ROI
	Wave W_ImageObjArea, M_Moments
	if (DimSize(W_ImageObjArea, 0)>0)	//	if there is no particle, like when the ROI is reset to 2 points
	//			the results returned in M_Moments are the columns: 0: the X-center of the ellipse, 1: the Y-center of the ellipse, 2: the major axis, 3: the minor axis, and 4: the angle (radians) that the major axis makes with the X-direction.
		W_ROI_Area[V_NumROI]=W_ImageObjArea[0]*V_Resolution*V_Resolution
		//	Save short and long radius*2 (= diameter) of the ellipse
		W_ROI_EllipseData[V_NumROI][0]=M_Moments[2]*V_Resolution*2; W_ROI_EllipseData[V_NumROI][1]=M_Moments[3]*V_Resolution*2
	
		//	Draw the ellipse on top of the ROI; done after SerVarNumROI which clears drawings on F_RM
		ControlInfo/W=P_ROI C_ShowFittedEllipse	//	V_Value=1 if checked
		if (V_Value)
			MydrawOvalInAxisCoordinates(M_Moments[2], M_Moments[3], W_ZonesROI[0][V_NumROI]+M_Moments[0], W_ZonesROI[1][V_NumROI]+M_Moments[1], -M_Moments[4], "MyEllipse", "F_RM", 1, "ProgFront", "top", "left", 2)
		endif
	endif
	
	SetDataFolder dfSav
End

//	Function to close the modal window and return control to BoutonGereROI
Function BoutonValideROI(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K F_DefineROI
	DoWindow/F F_RM
	GraphNormal/W=F_RM
End

//	Creates or suppresses the dependencies on ROI waves
Function DependencesROI(vChoix, vNumROI)
	variable vChoix, vNumROI

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NbChiffres
	String sNomDF1, sNomDF2, sFormule

	String NomROIx=AjouteNumero("ROIx", vNumROI, V_NbChiffres)
	String NomROIx0=AjouteNumero("ROIx0_", vNumROI, V_NbChiffres)
	String NomROIy=AjouteNumero("ROIy", vNumROI, V_NbChiffres)
	String NomROIy0=AjouteNumero("ROIy0_", vNumROI, V_NbChiffres)
	String NomROIdxyz=AjouteNumero("ROIdxyz", vNumROI, V_NbChiffres)	

	Wave/SDFR=dfROI/Z wX=$NomROIx, wY=$NomROIy, wX0=$NomROIx0, wY0=$NomROIy0, wDXYZ=$NomROIdxyz
	
	if ((WaveExists(wX0)) && (waveExists(wY0)) && (Waveexists(wDXYZ)))
		If (vChoix)
			sNomDF1=GetDataFolder(1, dfROI)
			sNomDF2=GetDataFolder(1, dfRatio)
			sFormule=sNomDF1+NomROIx0+"+"+sNomDF1+NomROIdxyz+"["+sNomDF2+"V_NumCoucheCourante][0]"
			SetFormula wX, sFormule
			sFormule=sNomDF1+NomROIy0+"+"+sNomDF1+NomROIdxyz+"["+sNomDF2+"V_NumCoucheCourante][1]"
			SetFormula wY, sFormule
		else
			SetFormula wX, ""
			SetFormula wY, ""
			wX=wX0; wY=wY0
		endif
	endif
End

//________________________________________________________________________________________________
//************************************************************************************************
// *** Displaying ROIs and traces ***

//	Handles the display of a ROI and associated traces with the right settings (tag, color, thickness...)
Function MaJApparenceROI(i, j)
	variable i, j
	variable vTest=0
	
	//	i=ROI #; j=what to do with the ROI
	//	Starting at 1 executes everything, to be on the safe side, but can take longer.
	//	Starting midway can go significantly faster, like case 4 ; to speed up ROI update when scrolling Z axis in a stack
	//	j=0 or j=1: remove ROI plot from F_RM and traces from F_TracesTriees; then plots again, but from this datafolder
	//	if j=0, exits just after removing all traces and ROI plots
	//	j=2: update color and thickness
	//	j=3: displays or hide traces, depending on tag selection and W_AfficheTrace; should not happen
	//	j=4: displays or hide ROI plots, depending on tag selection and W_AfficheROI
	
	//	Test if the relevant windows exist. If any one of the 3 required is missing, stops
	DoWindow F_Traces; vTest+=V_Flag
	DoWindow F_TracesTriees; vTest+=V_Flag
	DoWindow F_RM; vTest+=V_Flag
	if (vTest != 3)
		return(0)
	endif

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Temps_min
	NVAR/SDFR=dfRatio V_NbChiffres
	
	Wave/SDFR=dfROI W_CouleurTrace, W_CouleurROI, W_EpaisseurTrace, W_EpaisseurROI, W_Tag, W_AfficheROI, W_AfficheTrace

	String sNomROIx, sNomROIy, sNomTraceA, sNomTraceB, sNomTraceR, sNomAxe, sNomMasque
	sNomROIx=AjouteNumero("ROIx", i, V_NbChiffres)
	sNomROIy=AjouteNumero("ROIy", i, V_NbChiffres)
	sNomTraceA=AjouteNumero("TraceA", i, V_NbChiffres)
	sNomTraceB=AjouteNumero("TraceB", i, V_NbChiffres)
	sNomTraceR=AjouteNumero("TraceR", i, V_NbChiffres)
	sNomMasque=AjouteNumero("P_Masque", i, V_NbChiffres)
	sNomAxe="tag"+Num2Str(W_Tag[i])
	if (W_Tag[i]==0) 
		sNomAxe="left"
	endif

	Switch(j)
		case 0:
		case 1:
			//	Remove all traces with this number
			EnleveTrace("F_Traces", sNomTraceA)	//	This function repeats the removal of the trace until all traces with this name are gone
			EnleveTrace("F_Traces", sNomTraceB)
			EnleveTrace("F_Traces", sNomTraceR)
			EnleveTrace("F_TracesTriees", sNomTraceR)
			EnleveTrace("F_Pile", sNomROIy)
			EnleveTrace("F_RM", sNomROIy)
			if (j==0)
				break
			endif

			//	Appends the traces with this number to the 2 graphs.
			AppendToGraph/W=F_Traces/L=F_A dfROI:$sNomTraceA vs W_Temps_min
			AppendToGraph/W=F_Traces/L=F_B dfROI:$sNomTraceB vs W_Temps_min
			if (W_Tag[i]>=0 ) //	Only plots traces which are not tag=-1, ie do not plot traces of background regions
				AppendToGraph/W=F_Traces/L=left dfROI:$sNomTraceR vs W_Temps_min
				AppendToGraph/W=F_TracesTriees/L=$sNomAxe dfROI:$sNomTraceR vs W_Temps_min
			endif
			//	Plots ROI over image
			AppendToGraph/W=F_RM/L=left/T=top dfROI:$sNomROIy vs dfROI:$sNomROIx
			//	Plots ROI over Z stack, if present
			DoWindow F_Pile
			if (V_Flag)
				AppendToGraph/W=F_Pile/L=left/T=top dfROI:$sNomROIy vs dfROI:$sNomROIx
			endif
		//break	//	no break is intentional, to execute the following steps
		case 2:
			//	If a ROI color is too light, puts the corresponding trace in black
			if (W_CouleurROI[i][0]+W_CouleurROI[i][1]+W_CouleurROI[i][2]>55000*3)
				W_CouleurTrace[i][]=0
			else
				W_CouleurTrace[i][]=W_CouleurROI[i][q]	//	Make trace color same as ROI color
			endif
			
			//	Apply color and thickness to the ROI drawn on image (and stack, if present)
			ControlInfo/W=P_Controle C_AfficheROI	//	case est cochée -> V_Value=1
			ModifyGraph/Z/W=F_RM rgb($sNomROIy)=(W_CouleurROI[i][0],W_CouleurROI[i][1],W_CouleurROI[i][2])
			ModifyGraph/Z/W=F_RM lSize($sNomROIy)=W_EpaisseurROI[i]
			DoWindow F_Pile
			if (V_flag)
				ModifyGraph/Z/W=F_Pile rgb($sNomROIy)=(W_CouleurROI[i][0],W_CouleurROI[i][1],W_CouleurROI[i][2])
				ModifyGraph/Z/W=F_Pile lSize($sNomROIy)=W_EpaisseurROI[i]
			endif
			
			// Apply color and thickness to traces
			ModifyGraph/Z/W=F_Traces rgb($sNomTraceA)=(W_CouleurTrace[i][0],W_CouleurTrace[i][1],W_CouleurTrace[i][2])
			ModifyGraph/Z/W=F_Traces lSize($sNomTraceA)=W_EpaisseurTrace[i]	// épaisseur du trait
			ModifyGraph/Z/W=F_Traces mrkThick($sNomTraceA)=W_EpaisseurTrace[i]	//	épaisseur du trait appliquée au symbole, s'il existe
			ModifyGraph/Z/W=F_Traces rgb($sNomTraceB)=(W_CouleurTrace[i][0],W_CouleurTrace[i][1],W_CouleurTrace[i][2])
			ModifyGraph/Z/W=F_Traces lSize($sNomTraceB)=W_EpaisseurTrace[i]	// épaisseur du trait
			ModifyGraph/Z/W=F_Traces mrkThick($sNomTraceB)=W_EpaisseurTrace[i]	//	épaisseur du trait appliquée au symbole, s'il existe
			ModifyGraph/Z/W=F_Traces rgb($sNomTraceR)=(W_CouleurTrace[i][0],W_CouleurTrace[i][1],W_CouleurTrace[i][2])
			ModifyGraph/Z/W=F_Traces lSize($sNomTraceR)=W_EpaisseurTrace[i]	// épaisseur du trait
			ModifyGraph/Z/W=F_Traces mrkThick($sNomTraceR)=W_EpaisseurTrace[i]	//	épaisseur du trait appliquée au symbole, s'il existe
			ModifyGraph/Z/W=F_TracesTriees rgb($sNomTraceR)=(W_CouleurTrace[i][0],W_CouleurTrace[i][1],W_CouleurTrace[i][2])
			ModifyGraph/Z/W=F_TracesTriees lSize($sNomTraceR)=W_EpaisseurTrace[i]
		//	no break, to carry on with next step
		
		case 3:	// show or hide traces depending on group and W_AfficheTrace
//			variable aff=abs((W_AfficheTrace[i]==1) && (TesteGroupe(W_Tag[i]))-1)	//	j'ai utilisé W_AfficheTrace pour masquer artificiellement une trace, mais ça semble une mauvaise idée
			variable aff=abs(TesteGroupe(W_Tag[i]) - 1)		//	aff is 0 when the trace should be shown
			ModifyGraph/Z/W=F_TracesTriees hideTrace($sNomTraceR)=(aff)
			//	Background trace
			if (W_Tag[i] == -1)
				ModifyGraph/Z/W=F_Traces hideTrace($sNomTraceR)=1	//	the ratio trace of a background should not be shown
				ModifyGraph/Z/W=F_Traces hideTrace($sNomTraceA)=0	//	the intensity traces should be shown
				ModifyGraph/Z/W=F_Traces hideTrace($sNomTraceB)=0
			else
				aff=abs(TesteGroupe(W_Tag[i]) - 1)
				ModifyGraph/Z/W=F_Traces hideTrace($sNomTraceR)=aff	//	the ratio trace of a background can be shown if the group is displayed
				ModifyGraph/Z/W=F_Traces hideTrace($sNomTraceA)=aff
				ModifyGraph/Z/W=F_Traces hideTrace($sNomTraceB)=aff
			endif
		//break	//	No break, same reason

		case 4:
			variable a, b, c
			a=W_AfficheROI[i]
			b=TesteGroupe(W_Tag[i])
			c=(W_AfficheROI[i]==1) && (TesteGroupe(W_Tag[i]))
			aff=abs(c-1)
			ModifyGraph/Z/W=F_RM hideTrace($sNomROIy)=(aff)
			DoWindow F_Pile
			if (V_Flag)
				ModifyGraph/Z/W=F_Pile hideTrace($sNomROIy)=(aff)
			endif
		break
		default:
			DoAlert 0, "Case "+num2str(j)+" not treated in MajApparenceROI"
		break
	EndSwitch
End

//	Repeat the display update for all ROIs
Function MaJApparenceToutesROI(j)
	variable j
	variable i
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	NVAR/SDFR=dfROI V_NbROI
	for (i=1;(i<=V_NbROI);i+=1)
		MaJApparenceROI(i,j)
	endfor
End

Function SetVarNumROI(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	String sNomTrace, sNomTraceA, sNomTraceB, sNomROIy, sNomMasque2

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfROI W_PlanROI, W_PlanROIPileExterne, W_CouleurROI, W_EpaisseurROI, W_AfficheROI, D_Masque, D_Masque2, W_Tag, W_IntensiteOK
	NVAR/SDFR=dfROI V_NumROI, V_NbROI
	NVAR/SDFR=dfRatio V_NumPlan, V_NbChiffres

	//	Show the plane in the stack that corresponds to the plane of this ROI
	V_NumPlan=W_PlanROI[V_NumROI]
	GlissierePlan("rien",V_NumPlan,0)

	Button C_CouleurROI win=P_Controle, fcolor=(W_CouleurROI[ V_NumROI][0],W_CouleurROI[ V_NumROI][1],W_CouleurROI[ V_NumROI][2])
	
	ControlInfo/W=P_ROI C_AutoZoomROI
	if (V_Value)		//	If autozoom is activated
		CheckZoom("C_AutoZoomROI",1)
	endif
	
	MajApparenceToutesROI(2)	//	updates traces' appearance
	sNomTrace=AjouteNumero("TraceR", V_NumROI, V_NbChiffres)
	sNomTraceA=AjouteNumero("TraceA", V_NumROI, V_NbChiffres)
	sNomTraceB=AjouteNumero("TraceB", V_NumROI, V_NbChiffres)
	sNomROIy=AjouteNumero("ROIy", V_NumROI, V_NbChiffres)
	sNomMasque2=AjouteNumero("Masque2_", V_NumROI, V_NbChiffres)
	ModifyGraph/Z/W=F_Traces lSize($sNomTrace)=4
	ModifyGraph/Z/W=F_TracesTriees lSize($sNomTrace)=4
	//	Would be nice to put the selected trace on the forefront; to be done
//	ReorderTraces/W=F_TracesTriees 
	ModifyGraph/Z/W=F_Traces lSize($sNomTraceA)=4
	ModifyGraph/Z/W=F_Traces lSize($sNomTraceB)=4
	ModifyGraph/Z/W=F_RM hideTrace($sNomROIy)=(0)
	DoWindow F_Pile
	if (V_Flag)
		ModifyGraph/Z/W=F_Pile hideTrace($sNomROIy)=(0),  lSize($sNomROIy)=(4)
	endif
	ControlInfo/W=P_Controle C_AutoZoomROI
	if (!V_Value)
		ModifyGraph/Z/W=F_RM lSize($sNomROIy)=4		//if not in autozoom mode, increases the thickness of the ROI plot
	endif
	
	//	To display the mask of this ROI in P_ROI
	Wave wMasque2=dfROI:$sNomMasque2
	Duplicate/O wMasque2, D_Masque
	
	SetDrawLayer/W=F_RM/K ProgFront	//	to clear drawings such as ellipe

	//	Update SetVariable controls in P_Controle and optional F_Pile P_ROI
	SetVariable C_NumROI win=P_Controle,value=V_NumROI,limits={1,V_NbROI,1}
	SetVariable C_Tag win=P_Controle,value=W_Tag[V_NumROI]
	SetVariable/Z C_Tag win=P_ROI,value=W_Tag[V_NumROI]
	SetVariable/Z C_NumROI win=F_Pile,value=V_NumROI,limits={1,V_NbROI,1}
	SetVariable/Z C_Tag win=F_Pile,value=W_Tag[V_NumROI]
	SetVariable/Z C_PlanOptimal win=F_Pile,value=W_PlanROIPileExterne[V_NumROI]
	CheckBox/Z C_IntensiteOK win=P_ROI,value=W_IntensiteOK[V_NumROI]
	CheckBox/Z C_IntensiteOK win=F_Pile,value=W_IntensiteOK[V_NumROI]

	DoWindow P_ROI
	if (V_flag)
		AutoPositionWindow/M=0/R=F_RM P_ROI
	endif
	
	//	To clear the mask that shows thresholding for Sensor Concentration
	D_Masque2=NaN
	
End

//	Respond to checkboxes that define whether a group should be displayed or not
//	Stores the state of each of the 4 axes, bitwise
Function CheckAfficheGroupe(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF
	DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_TagsAffiches

	Variable i=-1	//	The function can be called with this parameter just to update axes and display
	String NomAxe
	
	StrSwitch(ctrlName)
		case "C_Groupe0":
			i=0
		break
		case "C_Groupe1":
			i=1
		break
		case "C_Groupe2":
			i=2
		break
		case "C_Groupe3":
			i=3
		break
	EndSwitch
	if (i>=0)
		NomAxe="tag"+Num2Str(i)
		if (i==0) 
			NomAxe="left"
		endif
		if (checked)
			ModifyGraph/W=F_TracesTriees noLabel($NomAxe)=0, axThick($NomAxe)=1
		else
			ModifyGraph/W=F_TracesTriees noLabel($NomAxe)=2, axThick($NomAxe)=0
		endif
	endif

	if (checked==1)
		V_TagsAffiches=V_TagsAffiches | (2^i)	//	set bit
	else
		V_TagsAffiches=V_TagsAffiches & ~(2^i)	//	clear bit
	endif
	MaJApparenceToutesROI(1)
	SetVarEspaceEvt("",0,"","")
	ControlInfo/W=P_Controle C_AfficheMoy
	CheckAfficheMoy("",V_Value)
	DenombreGroupe()
End

// *** \\\to show or hide tag names ***
//	
Function CheckAfficheNomTag(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_AfficheNomTag
	StrSwitch(ctrlName)
		case "C_CheckNomTag":
			V_AfficheNomTag=-1
		break
		case "C_CheckNomTag0":
			V_AfficheNomTag=0
		break
		case "C_CheckNomTag1":
			V_AfficheNomTag=1
		break
	EndSwitch
	CheckBox C_CheckNomTag, value=V_AfficheNomTag==-1
	CheckBox C_CheckNomTag0, value=V_AfficheNomTag==0
	CheckBox C_CheckNomTag1, value=V_AfficheNomTag==1
	SetVarEspaceEvt(ctrlName, 0, "", "")
End

//	Returns 1 if the group should be displayed, 0 otherwise
Function TesteGroupe(i)
	variable i

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_TagsAffiches

	variable vResultat

	if ((V_TagsAffiches & (2^i))>0)
		vResultat=1
	else
		vResultat=0
	endif
	return(vResultat)
end

//	Calculates axis position to space them evenly. Includes a tunable margin above and below
//	Called as a setvar function by the setvar controls which define these margins
//	Show the tag name, as indicated in V_AfficheNomTag
Function SetVarEspaceEvt(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio vMargeHaut=V_MargeHaut, vMargeBas=V_MargeBas, V_AfficheNomTag
	Wave/SDFR=dfRatio/T W_NombreTagStr

	variable i, vNbAxes, vTailleAxe, vMin, vMax, vEspaceGraphe, vEspaceEntreTags=5
	string sNomAxe, sNomAxeD, sNomTag

	for (i=0; (i<=3); i+=1)
		vNbAxes+=TesteGroupe(i)
	endfor
	vEspaceGraphe=100-vMargeHaut-vMargeBas
	vTailleAxe=(vEspaceGraphe-(vNbAxes-1)*vEspaceEntreTags)/vNbAxes
	vMin=vMargeBas; vMax=vMargeBas+vTailleAxe

	for (i=0; (i<=3); i+=1)
		sNomAxe="tag"+Num2Str(i)
		if (i==0)
			sNomAxe="left"
		endif
		sNomTag="NomTag"+num2Str(i)
		if (TesteGroupe(i))
			ModifyGraph/W=F_TracesTriees axisEnab($sNomAxe)={vMin/100, vMax/100},noLabel($sNomAxe)=0//,axThick($sNomAxe)=1, 
			ModifyGraph/W=F_TracesTriees freePos($sNomAxe)=0
			Switch(V_AfficheNomTag)
				case -1:		//	tag name on the left
//					TextBox/W=F_TracesTriees/C/N=$sNomTag/V=(1)/A=LB/B=1/F=0/X=(0)/Y=(vMin-40) W_NombreTagStr[i]			
					TextBox/W=F_TracesTriees/C/N=$sNomTag/V=(1)/A=LC/B=1/X=(3)/Y=(vMin-40) W_NombreTagStr[i]
				break
				case 0:		//	no tag name
					TextBox/W=F_TracesTriees/C/N=$sNomTag/V=0 W_NombreTagStr[i]
				break
				case 1:		//	tag name on the right
					TextBox/W=F_TracesTriees/C/N=$sNomTag/V=(1)/A=RC/B=1/X=(00)/Y=(vMin-40) W_NombreTagStr[i]
				break
			EndSwitch
			vMin=vMax+vEspaceEntreTags
			vMax=vMin+vTailleAxe
		else
			TextBox/W=F_TracesTriees/C/N=$sNomTag/V=0 W_NombreTagStr[i]		//	hide tag name
		endif
	endfor
	SetAxis/W=F_TracesTriees right 0,1	
End

//	Called by buttons to control ROI / traces coloring
Function BoutonCouleurs(ctrlname) : ButtonControl
	string ctrlname
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI  V_NumROI, V_NbROI
	Wave/SDFR=dfROI W_CouleurROI, W_CouleurTrace, W_Tag, W_IntensiteOK
	Wave M_CouleursVariees=root:z_Initialisation:f_Global:M_CouleursVariees

	variable i, j, index, vMin, vMax, vNbCouleurs
	variable vVoie=1	//	what data channel is used for intensity display; default is B channel
	
	StrSwitch(ctrlName)
		case "C_CouleursVariees":
			for(i=1; i<=V_NbROI; i+=1)
				index = mod(i, DimSize(M_CouleursVariees,0))				// Wrap after reaching the last color
				W_CouleurROI[i][]=M_CouleursVariees[index][q]
				W_CouleurTrace[i][]=M_CouleursVariees[index][q]
			endfor
		break
		//	Gives a color as a function of biosensor intensity in the ROI
		case "C_CouleursTriees":
			WaveStats/M=1/Q W_IntensiteOK
			if (V_Sum<3)
				DoAlert 0, "There should be more valid intensity values"
				return(0)
			endif
			Wave/SDFR=dfROI W_Intensite
			//	Here, should do something to make sure that intensity have been calculated
//			for (j=1; (j<=V_NbROI);j+=1)
//				if (TesteGroupe(W_Tag[j]))
//					wIntensite[j]=imag(MesureIntensite(j))	//	Get intensity from B channel
//				endif
//			endfor
			WaveStats/M=1/Q/PCST W_Intensite
			Wave M_WaveStats
			vMin=M_WaveStats[10][vVoie]
			vMax=M_WaveStats[12][vVoie]
			//	Convert the standard rainbow color table into a 2D wave
			ColorTab2Wave Rainbow
			Wave M_colors; vNbCouleurs=DimSize(M_colors, 0)-1
			for (j=1; (j<=V_NbROI);j+=1)
				if (TesteGroupe(W_Tag[j]))
					index=trunc((vMax-W_Intensite[j][vVoie])/(vMax-vMin)*vNbCouleurs)
					if (NumType(index)==0)
						W_CouleurROI[j][]=M_colors[index][q]; W_CouleurTrace[j][]=M_colors[index][q]
					else
						W_CouleurROI[j][]=0; W_CouleurTrace[j][]=0
					endif
				endif
			endfor
			KillWaves M_colors, M_WaveStats
		break
		case "C_CouleurROI":
			ChooseColor/C=(W_CouleurROI[V_NumROI][0],W_CouleurROI[V_NumROI][1],W_CouleurROI[V_NumROI][2])
			if (V_Flag==1)
				W_CouleurROI[V_NumROI][0]=V_Red
				W_CouleurROI[V_NumROI][1]=V_Green
				W_CouleurROI[V_NumROI][2]=V_Blue
			endif
			Button C_CouleurROI win=P_Controle, fcolor=(W_CouleurROI[V_NumROI][0],W_CouleurROI[V_NumROI][1],W_CouleurROI[V_NumROI][2])
		break
	endSwitch
	MaJApparenceToutesROI(2)
End

//	Changes the display of traces as markers or points
Function CheckROIMarqueurs(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVar/SDFR=dfRatio V_NbChiffres
	NVAR/SDFR=dfROI V_NbROI
	Wave/SDFR=dfROI W_CouleurTrace, W_CouleurROI, W_EpaisseurTrace

	Variable i, mode, size
	if (checked==1)
		StrSwitch(ctrlName)
			case "C_ROIMarqueurs":
				mode=4
			break
			case "C_ROI_Points":
				mode=2
				size=2
			break
		EndSwitch
	else
		mode=0
		size=1
	endif
	String NomROIy, NomTraceA, NomTraceB, NomTraceR
	for(i=1;(i<=V_NbROI);i+=1)
		// applique la couleur aux traces
		NomTraceA=AjouteNumero("TraceA", i, V_NbChiffres)
		NomTraceB=AjouteNumero("TraceB", i, V_NbChiffres)
		NomTraceR=AjouteNumero("TraceR", i, V_NbChiffres)
		StrSwitch(ctrlName)
			case "C_ROIMarqueurs":
				ModifyGraph/Z/W=F_Traces mode($NomTraceA)=mode, marker($NomTraceA)=8
				ModifyGraph/Z/W=F_Traces mode($NomTraceB)=mode, marker($NomTraceB)=8
				ModifyGraph/Z/W=F_Traces mode($NomTraceR)=mode, marker($NomTraceR)=8
			break
			case "C_ROI_Points":
//				ModifyGraph/Z/W=F_TracesTriees lsize($NomTraceR)=2, mode($NomTraceR)=mode
				ModifyGraph/Z/W=F_TracesTriees lsize($NomTraceR)=W_EpaisseurTrace[i], mode($NomTraceR)=mode, lsize($NomTraceR)=size
				ModifyGraph/Z/W=F_Traces mode($NomTraceR)=mode, marker($NomTraceR)=mode
			break
		EndSwitch
	endfor
end

// *** To display ROIs in a group with preset name and color ***
Function PopMenuPreReglagesROI(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName
	Variable popNum
	String popStr
	variable i, j, k

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfROI W_Tag, W_CouleurTagROI, W_CouleurTagTraces, W_CouleurTrace, W_CouleurROI
	NVAR/SDFR=dfROI V_NbROI
	Wave/SDFR=dfRatio/T W_NombreTagStr

	StrSwitch(popStr)
		case "D1/D2":	//	D1/D2 work : D1 in green, D2 in blue
			W_NombreTagStr[2]="D2"
			W_NombreTagStr[3]="D1"
			W_CouleurTagROI[0][0]=32767;W_CouleurTagROI[0][1]=32767;W_CouleurTagROI[0][2]=32767	//	Gris
			W_CouleurTagROI[1][0]=40000;W_CouleurTagROI[1][1]=20500;W_CouleurTagROI[1][2]=0		//	Orange
			W_CouleurTagROI[2][0]=0;W_CouleurTagROI[2][1]=13000;W_CouleurTagROI[2][2]=53000		//	Bleu
			W_CouleurTagROI[3][0]=0;W_CouleurTagROI[3][1]=40000;W_CouleurTagROI[3][2]=0				//	Vert
			W_CouleurTagTraces=W_CouleurTagROI
		break
		case "Heart":
			W_NombreTagStr=""
			W_NombreTagStr[1]="AT"
			W_CouleurTagROI[1][0]=65535;W_CouleurTagROI[1][1]=16385;W_CouleurTagROI[1][2]=16385
			W_NombreTagStr[2]="VT"			
			W_CouleurTagROI[2][0]=60672;W_CouleurTagROI[2][1]=60672;W_CouleurTagROI[2][2]=60672
			W_CouleurTagTraces=W_CouleurTagROI
		break
		case "ChIN":
			W_NombreTagStr[0]="MSN"
			W_NombreTagStr[3]="ChIN"
			W_CouleurTagROI[0][0]=0;W_CouleurTagROI[0][1]=0;W_CouleurTagROI[0][2]=50000	//	bleu
			W_CouleurTagROI[1][0]=50000;W_CouleurTagROI[1][1]=0;W_CouleurTagROI[1][2]=0		//	rouge
			W_CouleurTagTraces=W_CouleurTagROI
		break
		case "0":
		case "1":
		case "2":
		case "3":
			i=Str2Num(popStr)
			ChooseColor/C=(W_CouleurTagROI[i][0],W_CouleurTagROI[i][1],W_CouleurTagROI[i][2])
			if (V_Flag==1)	//	user selected a color, which is saved per tag
				W_CouleurTagROI[i][0]=V_Red
				W_CouleurTagROI[i][1]=V_Green
				W_CouleurTagROI[i][2]=V_Blue
			endif
			W_CouleurTagTraces[i][]=W_CouleurTagROI[i][q]
		break
	EndSwitch
	for (j=1; (j<=V_NbROI);j+=1)
		k=W_Tag[j]
		if (K>=0)
			W_CouleurTrace[j][]=W_CouleurTagTraces[k][q]
			W_CouleurROI[j][]=W_CouleurTagROI[k][q]
		endif
	endfor
	MaJApparenceToutesROI(2)
	CheckAfficheGroupe("",1)
End

//	*****************************************************************
// *** Utilities to handle ROI : counting them, placing them in a group... ***

//	Returns the # of the ROI with highest number; 0 if none.
Function NbROI()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfRatio V_NbChiffres
	NVAR/SDFR=dfROI V_NbROI
	
	Variable i=0
	Do
		i+=1
		Wave/Z/SDFR=dfROI wTest=$AjouteNumero("ROIx", i, V_NbChiffres)
	While (WaveExists(wTest))
	V_NbROI=i-1
	Return i-1
End

//	Called to change the tag corresponding to a ROI
Function SetVarTag(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_NumROI
	Wave/SDFR=dfROI W_Tag, W_CouleurROI

	if (varNum==-1)
		DoAlert 1, "Use this ROI as background?"
		if (V_Flag!=1)
			W_Tag[V_NumROI]=0
			return(0)
		endif
		MaJ_CasesACocher("Tous", 0, "", "")
	endif
	MaJApparenceROI(V_NumROI,1)
	Button C_CouleurROI win=P_Controle, fcolor=(W_CouleurROI[V_NumROI][0],W_CouleurROI[V_NumROI][1],W_CouleurROI[V_NumROI][2])
	DenombreGroupe()
end	

//	Counts the number of ROIs in each tag group
//	The number of ROI per group is stored in W_NombreTag
Function DenombreGroupe()
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Variable j
	SVAR/SDFR=dfROI S_Fonds
	NVAR/SDFR=dfROI V_NbFonds, V_NbROI
	V_NbFonds=0
	S_Fonds="Background ROIs n° :"
	Wave/SDFR=dfROI W_Tag
	Wave/SDFR=dfRatio W_NombreTag
	W_NombreTag=0
	for (j=1; (j<=V_NbROI);j+=1)
		if (W_Tag[j]>=0)	//	background (tag=-1) are not counted
			W_NombreTag[(W_Tag[j])]+=1
		endif
		if (W_Tag[j]==-1)
			V_NbFonds+=1
			S_Fonds+=num2str(j)+", "
		endif
	endfor
	if (V_NbFonds==0)
		S_Fonds="No background subtraction"
	endif
end

//	*****************************************************************
// *** Average traces ***

//	Calculates and optionally displays the average of all traces in a group
Function CheckAfficheMoy(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio W_Temps_min, W_NombreTag
	NVar/SDFR=dfRatio V_NbChiffres, Dim, kMax
	Wave/SDFR=dfROI W_Tag
	NVAR/SDFR=dfROI/Z V_NbROI, V_EpaisseurMoy, V_SD_ou_SEM

	Variable i, j
	String sNomTrace, sNomTraceMoy, sNomTraceSD, sNomTraceSEM, sNomErreur, sNomAxe

	//	Default is unchecked; start from clean situation
	//	remove averages that may already be displayed
	for  (j=0; (j<=3);j+=1)
		sNomTrace=AjouteNumero("TraceMoy", j, 1)
		EnleveTrace("F_TracesTriees", sNomTrace)
	endfor
	CheckBox/Z C_AfficheMoy win=P_Controle, value=0

	if (checked)
		//	This function can be called by something else that the checkbox control. Here, make sure that the control is checked
		CheckBox/Z C_AfficheMoy win=P_Controle, value=1

		//	Update W_NombreTag
		DenombreGroupe()
		
		//	Calculate the average traces
		for (i=0; (i<=3); i+=1)	// for each group
			sNomTraceMoy="TraceMoy"+Num2iStr(i)	//	name of avereage wave
			sNomTraceSD="TraceMoy"+"_SD"+Num2iStr(i)
			sNomTraceSEM="TraceMoy"+"_SEM"+Num2iStr(i)
			Make/O/N=(kMax) dfROI:$sNomTraceMoy
			Wave wMoy=dfROI:$sNomTraceMoy
			Make/O/N=(kMax) dfROI:$sNomTraceSD	
			Wave wSD=dfROI:$sNomTraceSD
			Make/O/N=(kMax) dfROI:$sNomTraceSEM
			Wave wSEM=dfROI:$sNomTraceSEM
	
			//	Calculates the average
			wMoy=0
			for (j=1; (j<=V_NbROI);j+=1)
				if (W_Tag[j]==i)	//	this trace belongs to the current group
					sNomTrace=AjouteNumero("TraceR", j, V_NbChiffres)
					Wave wTrace=dfROI:$sNomTrace
					wMoy+=wTrace
				endif
			endfor	
			if (W_NombreTag[i]>=1)	
				wMoy/=W_NombreTag[i]
			endif
	
			//	Calculates SD and SEM
			wSD=0
			for (j=1; (j<=V_NbROI);j+=1)
				if (W_Tag[j]==i)
					sNomTrace=AjouteNumero("TraceR", j, V_NbChiffres)
					Wave wTrace=dfROI:$sNomTrace
					wSD+=(wTrace-wMoy)^2
				endif
			endfor
			wSD/=(W_NombreTag[i]-1)
			wSD=sqrt(wSD)
			wSEM=wSD/sqrt(W_NombreTag[i])

			if (TesteGroupe(i))	//	group is checked and should be displayed
				if (i==0) 
					sNomAxe="left"
				else
					sNomAxe="tag"+Num2Str(i)
				endif
			
				AppendToGraph/W=F_TracesTriees/L=$sNomAxe/C=(0,0,0) wMoy vs W_Temps_min
				ModifyGraph/W=F_TracesTriees lsize($sNomTraceMoy)=V_EpaisseurMoy
				Switch (V_SD_ou_SEM)
					case 0:
					break
					case 1:	//	display SD
						ErrorBars/W=F_TracesTriees $sNomTraceMoy SHADE= {0,0,(0,0,0,0),(0,0,0,0)},wave=(wSD,wSD)
					break
					case 2:	//	display SEM
						ErrorBars/W=F_TracesTriees $sNomTraceMoy SHADE= {0,0,(0,0,0,0),(0,0,0,0)},wave=(wSEM,wSEM)
					break
				endswitch
			endif
		endfor
	endif
End

Function CheckAfficheSDSEM(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_SD_ou_SEM

	StrSwitch(ctrlName)
		case "C_SDSEM":
			V_SD_ou_SEM=0
		break
		case "C_SD":
			V_SD_ou_SEM=1
		break
		case "C_SEM":
			V_SD_ou_SEM=2
		break
	EndSwitch
	CheckBox C_SDSEM, value=V_SD_ou_SEM==0
	CheckBox C_SD, value=V_SD_ou_SEM==1
	CheckBox C_SEM, value=V_SD_ou_SEM==2
	CheckAfficheMoy("",1)
End

Function SetVarEpaisseurMoy(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	CheckAfficheMoy("",1)	//	déclenche l'affichage de la moyenne
End



//	*****************************************************************
// *** ROI tracking ***

//	Testing if mobile ROIs has already been activated; returns the number of mobile ROIs if any, 0 otherwise. 
//	if n=-1: checks all ROIs in this analysis datafolder
//	otherwise, checks whether ROI# n has some movement with it.
Function TesteROIMobile(n)
	variable n
	variable i, vDebut, vFin, vResultat
	String NomROIdxyz
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_NbROI
	NVAR/SDFR=dfRatio V_NbChiffres
	
	if (n==-1)
		vDebut=1; vFin=V_NbROI	//	applies to all ROIs
	else 
		vDebut=n; vFin=n	//	applies only to ROI #n
	endif
	
	for (i=vDebut; (i<=vFin); i+=1)
		NomROIdxyz=AjouteNumero("ROIdxyz", i, V_NbChiffres)
		Wave/SDFR=dfROI/Z wDXYZ=$NomROIdxyz
		if (WaveExists(wDXYZ))
			WaveStats/M=1/Q wDXYZ
			if ((V_min!=0) || (V_max!=0))	//	detects if there is a non-zero value in the movement wave
				vResultat+=1
			endif
		endif
	endfor
	return(vResultat)
End

//	Respond to radio buttons on P_ROI
Function CheckBasculeBoutonRadio(ctrlName, checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_PropageDepROI

	StrSwitch(ctrlName)
//	radio buttons for autotracking
		case "C_CheckDepROI_Gauche":
			V_PropageDepROI=-1
		break
		case "C_CheckDepROI_Milieu":
			V_PropageDepROI=0
		break
		case "C_CheckDepROI_Droite":
			V_PropageDepROI=1
		break
	EndSwitch
	CheckBox C_CheckDepROI_Gauche, win=P_ROI, value=V_PropageDepROI==-1
	CheckBox C_CheckDepROI_Milieu, win=P_ROI, value=V_PropageDepROI==0
	CheckBox C_CheckDepROI_Droite, win=P_ROI, value=V_PropageDepROI==1
End

Function BoutonDeplaceROI(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_PropageDepROI, V_VoieROIMobile, V_Position0DeplaceROI, V_MargeROIMobile, V_NumROI
	WAVE/SDFR=dfRatio P_ACorr, P_BCorr
	NVAR/SDFR=dfRatio V_NumcoucheCourante, V_NbChiffres, Dim
	
	variable vQuelleVoie, j, dx, dy, dz
	String sNomROIdxyz=AjouteNumero("ROIdxyz", V_NumROI, V_NbChiffres), sNomTemp

	Wave/SDFR=dfROI W_ZonesROI, wROIdxyz=$sNomROIdxyz
	
	StrSwitch(ctrlName)
		case "C_EffaceDep":
			DoAlert/T="Warning" 1, "Clear movements for this ROI ?"
			if (V_Flag==1)
				wROIdxyz=0
			endif
		break
		case "C_AfficheDep":
			KillWindow/Z T_MouvementROI
			Edit/K=1/N=T_MouvementROI/W=(200+V_NumROI*10,44,460+V_NumROI*10,633) wROIdxyz as sNomROIdxyz
			ModifyTable format(Point)=1,width=50
			AutoPositionWindow/M=0/R=P_ROI T_MouvementROI
			Return(0)
		break
		case "C_CopieDep":
			prompt j, "# of the ROI which contains the movement to copy:"
			DoPrompt "Copy the movement from another ROI", j
			sNomTemp=AjouteNumero("ROIdxyz", j, V_NbChiffres)
			Wave/SDFR=dfROI/Z wTemp=$sNomTemp
			if (WaveExists(wTemp))
				wROIdxyz=wTemp
			else
				DoAlert 0, "This movement doesn't exist"
				Return(0)
			endif
		break
		case "C_PoursuiteAuto":
			SetDataFolder dfROI	//	so that the extracted images are stored in the current dfROI
			If (Numtype(V_Position0DeplaceROI)!=0)		//	If automatic ROI tracking has never been used, the reference time-point is unknown
				V_Position0DeplaceROI=V_NumcoucheCourante	//	Keeps the reference time-point
			endif
			Wave/Z P_Ref		//	Should contains the image stack at the reference time-point
			if (!WaveExists(P_Ref))	//	If not present, storing the reference time-point's image
				Prompt vQuelleVoie, "What data channel? Choose the one with best contrast:", popup, "A;B"
				DoPrompt "Automatic movement tracking "+num2str(V_NumcoucheCourante), vQuelleVoie
				if (V_Flag==1)	//	cancel
					return(0)
				endif
				V_VoieROIMobile=vQuelleVoie	//	A=1; B=2 etc...
				V_NumcoucheCourante=V_Position0DeplaceROI
				GereImages(V_NumcoucheCourante, 1, 2)	//	reads P_ACorr and P_BCorr for the current (reference) time-point
				Switch(V_VoieROIMobile)
					case 1:
						Duplicate/O P_ACorr, dfROI:P_Ref
						PopMenuAffichage("",0,"A")
					break
					case 2:
						Duplicate/O P_BCorr, dfROI:P_Ref
						PopMenuAffichage("",0,"B")
					break
				EndSwitch
			endif
			Wave P_Ref
			Make/N=6/FREE wZoneROI, wZoneAExtraireRef, wZoneAExtraireCorr
			wZoneROI=W_ZonesROI[p][V_NumROI]	//	zone encompassing the ROI
			wZoneAExtraireRef=wZoneROI
			//	Enlarge this zone by a number of pixels specified in V_MargeROIMobile
			wZoneAExtraireRef[0]-=V_MargeROIMobile
			wZoneAExtraireRef[1]-=V_MargeROIMobile
			wZoneAExtraireRef[3]+=V_MargeROIMobile
			wZoneAExtraireRef[4]+=V_MargeROIMobile
			//	Extract the reference zone from the reference image
			Wave P_Ref_Zone=ExtraitZone(wZoneAExtraireRef, P_Ref)	//	the name of the wave is given by ExtraitZone
			Duplicate/O P_Ref_Zone, P_Corr_Zone
			Fastop P_Corr_Zone=0
			DoWindow F_Ref
			if (V_Flag==0)
				NewImage/S=0/K=1/N=F_Ref P_Ref_Zone
				ModifyImage/W=F_Ref P_Ref_Zone, ctab= {*,*,Grays,0}
				ModifyGraph/W=F_Ref width={perUnit,1,top}, height={perUnit,1,left}, expand=2
				DoWindow/T F_Ref, "Reference"
				AutoPositionWindow/M=0/R=P_ROI F_Ref
			endif
			DoWindow F_Test
			if (V_Flag==0)
				NewImage/S=0/K=1/N=F_Test P_Corr_Zone
				ModifyImage/W=F_Test P_Corr_Zone, ctab= {*,*,Grays,0}
				ModifyGraph/W=F_Test width={perUnit,1,top}, height={perUnit,1,left}, expand=2
				DoWindow/T F_Test, "Test"
				AutoPositionWindow/M=1/R=F_Ref F_Test
			endif

			GereImages(V_NumcoucheCourante, 1, 2)	//	reads P_ACorr and P_BCorr for the current time-point
			//	determine the zone to extract from the current time-point
			wZoneAExtraireCorr=wZoneAExtraireRef	//	Zone in the current image is the same as in the ref...
			// ... + the movement of the ROI at the current time-point (can be already set manually, or propagated from previous time-point)
			wZoneAExtraireCorr[0]+=wROIdxyz[V_NumcoucheCourante][0]
			wZoneAExtraireCorr[1]+=wROIdxyz[V_NumcoucheCourante][1]
			wZoneAExtraireCorr[3]+=wROIdxyz[V_NumcoucheCourante][0]
			wZoneAExtraireCorr[4]+=wROIdxyz[V_NumcoucheCourante][1]
			//	Extract the zone from the correct image at the current time-point
			Switch(V_VoieROIMobile)
				case 1:
					Wave P_ACorr_Zone=ExtraitZone(wZoneAExtraireCorr, P_ACorr)
					Duplicate/O P_ACorr_Zone, P_Corr_Zone
				break
				case 2:
					Wave P_BCorr_Zone=ExtraitZone(wZoneAExtraireCorr, P_BCorr)
					Duplicate/O P_BCorr_Zone, P_Corr_Zone
				break
			EndSwitch
			DoUpdate

// *** Calculate ROI movement ***
			ImageRegistration/Q/CSNR=0/BVAL=1/CONV=0/FLVL=4/PRDL=4/TRNS={1,1,1}/ROT={0,0,0} testwave=P_Corr_Zone, refwave=P_Ref_Zone
			Wave W_RegParams
			dx=-round(W_RegParams[0])
			dy=-round(W_RegParams[1])
			dz=-round(W_RegParams[2])
		break
		case "C_Haut4":
			dy=-1
		break
		case "C_Bas4":
			dy=1
		break
		case "C_Gauche4":
			dx=-1
		break
		case "C_Droite4":
			dx=1
		break
		case "C_Haut5":
			dy=-5
		break
		case "C_Bas5":
			dy=5
		break
		case "C_Gauche5":
			dx=-5
		break
		case "C_Droite5":
			dx=5
		break
		case "C_HautZ4":
			dz=1
		break
		case "C_BasZ4":
			dz=-1
		break
	endswitch
	
	//	Add the delta value to the global movement wave
	wROIdxyz[V_NumcoucheCourante][0]+=dx
	wROIdxyz[V_NumcoucheCourante][1]+=dy
	wROIdxyz[V_NumcoucheCourante][2]+=dz

	//	if control is checked, propagate to earlier time-points
	ControlInfo/W=P_ROI C_CheckDepROI_Gauche
	if (V_Value)
		wROIdxyz[0,V_NumcoucheCourante-1][]=wROIdxyz[V_NumcoucheCourante][q]
	endif
	//	if control is checked, propagate to later time-points
	ControlInfo/W=P_ROI C_CheckDepROI_Droite
	if (V_Value)
		wROIdxyz[V_NumcoucheCourante+1, ][]=wROIdxyz[V_NumcoucheCourante][q]
	endif

	WaveStats/Q/M=1 wROIdxyz
	DependencesROI(( (V_min!=0) || (V_max !=0) ), V_NumROI)		//	establish the dependance if there is a movement, or clears it if there is no movement
	
	Init_P_ROI()	//	to update the status of ROI movement indicator

	//	to check the ROI measurement checkbox on P_Controle
	MaJ_CasesACocher("Mesure",0, "", "")	
End

//	Relay function that will call BoutonDeplaceROI("C_PoursuiteAuto") as many times as needed.
//	Handles the automatic repetition of ROI movement correction for a number or a single data point
Function BoutonDeplaceToutesROI(ctrlName) : ButtonControl
	String ctrlName

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_PropageDepROI, V_NumROI
	NVAR/SDFR=dfRatio V_NumcoucheCourante, Dim

	Do
		BoutonDeplaceROI("C_PoursuiteAuto")

		//	increment or decrement the current time-point depending on propagation
		V_NumcoucheCourante=V_NumcoucheCourante+V_PropageDepROI

		//	Determinds the end of the "Do-While" loop
		if ((V_NumcoucheCourante<=0) || (V_NumcoucheCourante>=Dim) || (V_PropageDepROI==0))
			V_NumcoucheCourante=(V_NumcoucheCourante<0) ? 0 : V_NumcoucheCourante
			V_NumcoucheCourante=(V_NumcoucheCourante>=Dim) ? Dim-1 : V_NumcoucheCourante
			KillWaves/Z P_ACorr_Zone, P_BCorr_Zone
			KillWindow/Z F_Ref
			KillWindow/Z F_Test
			SignalSonore(1)
			Break	//	End of the loop
		endif
	while (1)
End

//________________________________________________________________________________________________
//************************************************************************************************
// *** Perform the measurements over the ROI ***
//	Calculations are done on a sub-region of the images, which is defined in W_ZonesROI

Function MesureNouveau(i)
	variable i	//	ROI number

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/SDFR=dfRatio P_ACorr, P_BCorr, P_RatioxIMoy, P_IMoy, P_Ratio
	NVAR/SDFR=dfRatio j=V_NumCoucheCourante,  V_R0, V_NbChiffres, Dim, V_SensRatio, V_R0
	
	DFREF dfSav=GetDataFolderDFR()
	SetDataFolder dfROI
	
	Variable x0, y0, z0, x1, y1, z1

//	Create the name of the objects based on ROI number i
	String sNomTraceA, sNomTraceB, sNomTraceR, sNomMasque, sNomDep, sNomHisto
	Wave/SDFR=dfROI W_CouleurTrace, W_CouleurROI, W_EpaisseurTrace, W_EpaisseurROI, W_Tag, W_AfficheROI, W_AfficheTrace, W_PlanROI, W_Fond, W_ZonesROI
	sNomTraceA=AjouteNumero("TraceA", i, V_NbChiffres)
	sNomTraceB=AjouteNumero("TraceB", i, V_NbChiffres)
	sNomTraceR=AjouteNumero("TraceR", i, V_NbChiffres)
	sNomDep=AjouteNumero("ROIdxyz", i, V_NbChiffres)

	NVAR/SDFR=dfROI V_Histogramme
	NVAR/SDFR=dfROI V_ModeCalcul	//	=0 if calculating the ratio of average intensities; =1 if calculating the average ratio over the whole ROI (most common)
	Wave/SDFR=dfROI W_Tag, W_PlanROI
	Wave/SDFR=dfROI TraceA=$sNomTraceA, TraceB=$sNomTraceB, TraceR=$sNomTraceR
	sNomMasque=AjouteNumero("Masque2_",i, V_NbChiffres)
	Wave/SDFR=dfROI wMasque=$sNomMasque	//	at this stage, wMasque is the mask to use to do the measurement

//	Zone definition to perform the measurement
	Make/FREE/N=6 wZone
	wZone=W_ZonesROI[p][i]
	Wave/SDFR=dfROI wDep=$sNomDep		//	movement for this ROI
	variable dx=wDep[j][0], dy=wDep[j][1], dz=wDep[j][2]
	wZone[0]+=dx; wZone[1]+=dy; wZone[2]+=dz
	wZone[3]+=dx; wZone[4]+=dy; wZone[5]+=dz

//	Measure raw intensities with ImageStats
//	Measurement of the intensity is performed on a plane
	Wave pA=ExtraitZone(wZone, P_ACorr)	//	extract the zone around the ROI
	ImageStats /M=1/R=wMasque pA
	TraceA[j]=V_avg
	Wave pB=ExtraitZone(wZone, P_BCorr)
	ImageStats /M=1/R=wMasque pB
	TraceB[j]=V_avg

//	If ROI is defined as background,
	if(W_Tag[i]==-1)
		TraceR[j]=NaN	//	ratio is meaningless
		return(0)		//	terminate this function
	endif

	Switch(V_ModeCalcul)
		case 0:	//	ratio of average intensities
			if (V_SensRatio==1)
				TraceR[j]=TraceA[j]/TraceB[j]/V_R0
			else
				TraceR[j]=V_R0*TraceB[j]/TraceA[j]
			endif
		break
		case 1:	//	average of ratios weighted by intensity
//	Extract a zone from the image a zone encompassing the ROI -> calculation is a lot faster than on the whole image
			Wave pRatioxIMoy=ExtraitZone(wZone, P_RatioxIMoy)	//	get the piece of the RatioxIMoy "image"
			Wave pIMoy=ExtraitZone(wZone, P_IMoy)	//	same for IMoy "image"
			ImageStats /M=1/R=wMasque pRatioxIMoy
			TraceR[j]=V_avg	//	calculate < ratio x IMoy >
			ImageStats /M=1/R=wMasque pIMoy
			TraceR[j]/=V_avg	//	final value is < ratio x IMoy > / < IMoy >
		break
		case 2:	//	average of ratios NOT weighted by intensity
			Wave pRatio=ExtraitZone(wZone, P_Ratio)	//	get the piece of the Ratio "image"
			ImageStats /M=1/R=wMasque pRatio
			TraceR[j]=V_avg
		break
	EndSwitch
	
	//	Calculate the histogram of ratio values
	if (V_Histogramme)
		variable mini, maxi, delta, bin
		sNomHisto=AjouteNumero("Histo",i,V_NbChiffres)
		Make/O dfROI:$sNomHisto
		Wave pRatio=ExtraitZone(wZone, P_Ratio)
		WaveStats pRatio
		mini=trunc(V_min)
		maxi=trunc(V_max)+1
		delta=0.1
		bin=maxi-mini
		Histogram/DEST=dfROI:$sNomHisto/B={mini,delta,bin} pRatio
	endif
	SetDataFolder dfSav
end


//	Called when background correction is cancelled, by pressing the "cancel" button in A&B corrections sub-panel
Function BoutonEffaceFonds(ctrlName) : ButtonControl
	String ctrlName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	DFREF dfROI = W_DFREF[0]
	Variable j
	Wave/SDFR=dfROI W_Tag, W_Fond
	NVAR/SDFR=dfROI V_NbROI
	for (j=1; (j<=V_NbROI);j+=1)
		if (W_Tag[j]==-1)
			W_Tag[j]=0	//	on met à 0 le tag des régions qui étaient définies comme fond
		endif
	endfor
	W_Fond=0
	MaJApparenceToutesROI(1)
	MaJ_CasesACocher("Tous", 0, "", "")
End

//	Called by the P_TableauROI window, which displays the table with all ROI numbers
Function BoutonParamsROI(ctrlName) : ButtonControl
	String ctrlName
	variable i=1
	
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_NbROI
	Wave/SDFR=dfROI W_IntensiteOK, W_EpaisseurTrace, W_EpaisseurROI

	StrSwitch(ctrlName)
		case "C_ToutesIntensitesOK":
			W_IntensiteOK=0
			W_IntensiteOK[1,V_NbROI]=1
		break
		case "C_ToutesROI":
			prompt i, "thickness: "
			DoPrompt "Enter value", i
			W_EpaisseurROI[1,V_NbROI]=i
			MaJApparenceToutesROI(2)
		break
		case "C_ToutesTraces":
			prompt i, "thickness: "
			DoPrompt "Enter value", i
			W_EpaisseurTrace[1,V_NbROI]=i
			MaJApparenceToutesROI(2)
		break
	EndSwitch
End

//	Called by checkboxes that display ROIs.
Function CheckAfficheROI(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	NVAR/SDFR=dfROI V_NumROI
	Wave/SDFR=dfROI W_AfficheROI

	StrSwitch(ctrlName)
		case "C_AfficheROI":
		break
		case "C_AfficheROI_TousPlans":
			if (checked)
				CheckBox C_AfficheROI win=P_Controle, value=1
			endif
		break
	EndSwitch
	W_AfficheROI=checked
	MaJApparenceToutesROI(2)
End

// *** Intensity measurement - Sensor concentration  ***
//	THIS IS DIFFERENT FROM THE AVERAGE INTENSITY MEASURED OVER A AND B CHANNELS AND DISPLAYED AS TRACES
//	See the documentation for more explanation
//	Estimates the density of biosensor included in ROI #i; Save the result in W_Intensite
//	Depending on the global variable V_SourceIntensite, performs the measurement either on the current image pair or on an external image stack
//	Uses a threshold to include only the 10% brightest pixels
Function MesureIntensite(i)
	variable i	//	ROI #

	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]; DFREF dfRatio=W_DFREF[3]
	Wave/Z/SDFR=dfRatio P_ACorr, P_BCorr
	if (!WaveExists(P_ACorr))	//	can happen if ratio was not initialized
		Return(0)
	endif
	Wave/SDFR=dfROI W_PlanROI, W_ZonesROI, W_IntensiteOK, W_Intensite, D_Masque2, W_PlanROIPileExterne
	NVAR/SDFR=dfRatio V_NbChiffres, V_NumCoucheCourante, V_MoyUnifA, V_MoyUnifB
	NVAR/SDFR=dfROI V_SourceIntensite		//	-1=ratio data, A and B channels (default); 0=stack acquired in A channel; 1=stack acquired in B channel...
	NVAR/SDFR=dfROI V_FractionDesPixelsPourIntensite	//	fraction of pixels to keep in the ROI for estimating sensor concentration

	Variable IntensA, IntensB, Intens
	String sNomMasque, sNomDep
	Variable vFraction=0.1	//	Fraction of pixels with highest intensity, kept to estimate the average intensity over these pixels.

	if (W_IntensiteOK[i]==0)		//	This ROI is not validated for intensity measurement, -> return NaN
		W_Intensite[i][]=NaN; Return(0)
	endif
		
	//	Acessing the mask corresponding to this ROI
	sNomMasque=AjouteNumero("Masque2_",i, V_NbChiffres)
	Wave/SDFR=dfROI/Z wMasque=$sNomMasque	//	wMasque is the mask to be used for the measurement
	if (!WaveExists(wMasque))	//	abort if missing
		W_Intensite[i][]=NaN; Return(0)
	endif

	Make/FREE/N=6 wZone	//	Zone for measurement
	wZone=W_ZonesROI[p][i]
	
	//	movement of the zone
	sNomDep=AjouteNumero("ROIdxyz", i, V_NbChiffres)
	Wave/SDFR=dfROI wDep=$sNomDep	//	wave containing the movement of the ROI
	//	add the movement to the zone definition
	wZone[0]+=wDep[V_NumCoucheCourante][0]	//	x0
	wZone[1]+=wDep[V_NumCoucheCourante][1]	//	y0
	wZone[2]=0								//	z0
	wZone[3]+=wDep[V_NumCoucheCourante][0]	//	x1
	wZone[4]+=wDep[V_NumCoucheCourante][1]	//	y1
	wZone[5]=1								//	z1

//	Depending on the image source, perform the intensity measurement on the correct image
	Switch(V_SourceIntensite)
		case -1:	//	on current image pair
		//	Intensity measurement with ImageStats on the specified plane #
			Wave pA=ExtraitZone(wZone, P_ACorr)
			IntensA=Mesure1Intensite(pA, wMasque, D_Masque2, V_FractionDesPixelsPourIntensite)
			Wave pB=ExtraitZone(wZone, P_BCorr)
			IntensB=Mesure1Intensite(pB, wMasque, D_Masque2, V_FractionDesPixelsPourIntensite)
		break
		case 0:	//	on external stack taken on A or B channel
		case 1:
			Wave/SDFR=dfRatio/Z P_Pile
			If (WaveExists(P_Pile))
				ImageTransform/P=(W_PlanROIPileExterne[i]) getplane P_Pile
				Wave M_ImagePlane
				Redimension/N=(-1, -1, 1) M_ImagePlane
				Wave pA=ExtraitZone(wZone, M_ImagePlane)
				Intens=Mesure1Intensite(pA, wMasque, D_Masque2, V_FractionDesPixelsPourIntensite)
			else
				Intens=NaN	//	in case the stack has not been loaded, return NaN
			endif
			Switch(V_SourceIntensite)
				case 0:
					IntensA=Intens
					IntensB=NaN
				break
				case 1:
					IntensA=NaN
					IntensB=Intens
				break
			EndSwitch
			KillWaves M_ImagePlane		
		break
	endswitch

	//	Intensity is reported as a fraction of the Unif reference
	W_Intensite[i][0]=IntensA/V_MoyUnifA
	W_Intensite[i][1]=	IntensB/V_MoyUnifB
	KillWaves/Z pA, pB
End

//	Performs the intensity measurement
//	Returns the intensity measured over the vSeuil fraction of brightest pixels
//	Updates the D_Masque2 wave in the current datafolder to show the original image multiplied by the mask
Function Mesure1Intensite(wImage, wMasque, wMasque2, vSeuil)
	Wave wImage, wMasque, wMasque2
	variable vSeuil

	Variable vImin, vImax, vNbPixTot	//	Intensity min et max, and nb of pixels in the ROI
	Variable vNbPixThr
	Variable vDecrement	//	Intensity value to decrement in the loop

	WaveStats/M=1/Q wImage	//	image might be empty; in that case, return NaN
	if (V_max==0)
		Return(NaN)
	endif
	
	//	The standard Igor way of using masks in ImageStats is 0 for mask, 1 elsewhere; 
	//	here, the inverse is needed, ie 1 for the zone to measure, 0 for background
	Duplicate/O/FREE wMasque, wMasqueInverse
	wMasqueInverse=Abs(wMasque-1)	//	invert the mask

	//	Erase (set to 0) pixels of the image that don't belong to the mask
	wImage*=wMasqueInverse

	//	Determines the points with highest/lowest intensity and their number in the zone
	ImageStats/R=wMasque wImage
	vIMin=V_min; vIMax=V_max; vNbPixTot=V_npnts
	vDecrement=trunc((vIMax-vIMin)/30)
	if (vDecrement==0)
		vDecrement=1
	endif
	//	Thresholding : decrement max intensity until reaching 10% of pixels
	Do
		vIMax-=vDecrement
		ImageThreshold/Q/M=0/T=(vIMax) wImage
		Wave M_ImageThresh
		M_ImageThresh/=255		//	mask has values 0 or 255; change that to 0 or 1; 1 means good pixel
		vNbPixThr=Sum(M_ImageThresh)	//	number of good pixels
	While (vNbPixThr<vSeuil*vNbPixTot)	//	repeat the decrement until reaching 10% of pixels in the mask

	//	now, M_ImageThresh contains 10% most intense pixels
	//	Update the mask image to show the result
	Duplicate/O M_ImageThresh, wMasque2
	Redimension/S wMasque2
	Fastop wMasque2=wMasque2*wImage
	
	M_ImageThresh-=1	//	met à 0 la zone ROI
	ImageStats/M=1/R=M_ImageThresh wImage
	KillWaves M_ImageThresh
	Return(V_avg)
end

Function SetVarProc_SensorConcentration(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	NVAR/SDFR=dfROI V_NumROI	
	MesureIntensite(V_NumROI)
End

Function CheckProc_IntensiteOK(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	NVAR/SDFR=dfROI V_NumROI	
	Wave/sdfr=dfROI W_IntensiteOK, D_Masque2
	W_IntensiteOK[V_NumROI]=checked
	MesureIntensite(V_NumROI)
	if (W_IntensiteOK[V_NumROI]==0)		//	This ROI is not validated for intensity measurement, -> return NaN
		D_Masque2=NaN
	endif
End

Function BoutonProc_Intensite(ctrlName) : ButtonControl
	String ctrlName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	NVAR/SDFR=dfROI V_NbROI	
	variable i
	StrSwitch(ctrlName)
		case "C_MaJIntensite":
			for (i=1; (i<=V_NbROI); i+=1)
				MesureIntensite(i)
			endfor
		break
	EndSwitch
End

Function CheckProc_SourceIntensite(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	NVAR/SDFR=dfROI V_SourceIntensite
	StrSwitch(ctrlName)
		case "C_CheckSourceIntensiteDefault":
			V_SourceIntensite=-1
		break
		case "C_CheckSourceIntensiteExtA":
			V_SourceIntensite=0
		break
		case "C_CheckSourceIntensiteExtB":
			V_SourceIntensite=1
		break
	EndSwitch
	CheckBox C_CheckSourceIntensiteDefault win=P_ROI, value=V_SourceIntensite==-1
	CheckBox C_CheckSourceIntensiteExtA win=P_ROI, value=V_SourceIntensite==0
	CheckBox C_CheckSourceIntensiteExtB win=P_ROI, value=V_SourceIntensite==1
End

Function SetVarProc_PlanIntensite(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum
	String varStr
	String varName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	Wave/SDFR=dfROI W_PlanROIPileExterne
	NVAR/SDFR=dfROI V_NumROI

	StrSwitch(ctrlName)
		case "C_NumROI":
			ModifyImage P_Pile plane=W_PlanROIPileExterne[V_NumROI]
			SetVarNumRoi(ctrlName,varNum,varStr,varName)
		break
	EndSwitch
End

Function BoutonProc_PlanOptimal(ctrlName) : ButtonControl
	String ctrlName
	Wave/DF W_DFREF=root:W_DFREF; DFREF dfROI=W_DFREF[0]
	NVAR/SDFR=dfROI V_NumROI	
	Wave/SDFR=dfROI W_PlanROIPileExterne
	ControlInfo C_Plan		//	to get the current plane selected by the slider
	W_PlanROIPileExterne[V_NumROI]=V_Value
End